// *********************  FILE:  driverE.cc  *******************************
//
// This file contains a program that drives the simulation of an mm1
// queuing system. The simulation may be run any number of times with
// the results written to a file that may be used for data analysis.
//
// Programmer:  J. Ingram
// Date:  January 22, 1997
// Updated documentation:  June 2, 1997 to show use of the simNode class
// for list nodes
// 
// An mm1 queuing system is a system, such as a bank with one teller or
// an information desk with one helper, in which there is one server and
// one queue for customers waiting to be served.  It is assumed that
// interarrival times for customers are independent and exponentially
// distributed and that the service times for the customers are also
// independent and exponentially distributed.  Such a system is modeled
// by the class mm1System which is derived from the generic class System.
// 
// Input to the program:  
//         the mean interarrival time for customers 
//         a stream number (1 - 100) for generating random interarrival times
//         the mean service time for customers
//         a stream number (1 - 100) for generating random service times
//         the number of customers to simulate
//
// Usage:  This program uses several classes whose declarations and
// definitions are in separate files.  A brief description of the classes:
//
//       Class mm1System models an mm1 queuing system and is derived from
//            the base class System which contains the data and function
//            members for modeling a generic system (properties common to
//            all discrete event simulations of systems) -- in files
//            system.h, system.cc, and mm1.cc
//       Class simNode is a class for the basic list nodes used by the
//            event list (of type OrderedList) and the queues (type Queue) --
//            in files snode.h and snode.cc
//       Class Queue is a utility classes for modeling queues -
//            in files queue.h and queue.cc
//       Classes OrderedList is a utility class for handling
//            the event list (an ordered list of simNodes) in the simulation -
//            in files olist.c and olist.cc
//       Class RandNum is a class used for random number generation - in
//            files rand.h and rand.cc
//
// The following Makefile shows the dependency among the files:
//
// GCC = g++
//
// runmm1:	        rand.o snode.o queue.o olist.o system.o mm1.o driver.o
//		$(GCC) -o $@ -lm driver.o rand.o snode.o queue.o olist.o \
//                             system.o mm1.o
//
// runss:	        rand.o snode.o queue.o olist.o system.o mm1.o driverE.o
//		$(GCC) -o $@ -lm driverE.o rand.o snode.o queue.o olist.o \
//                             system.o mm1.o
//
// rand.o:	        rand.cc rand.h
//		$(GCC)  -c rand.cc
//
// snode.o:             snode.cc snode.h
//              $(GCC)  -c snode.cc
// 
// queue.o:		queue.cc queue.h snode.h
//		$(GCC)  -c  queue.cc
//
// olist.o:		olist.cc olist.h snode.h
//		$(GCC)  -c olist.cc
//
// system.o:		system.cc system.h snode.h olist.h rand.h queue.h
//		$(GCC)  -c system.cc
//
// mm1.o:		 mm1.cc mm1.h system.h snode.h olist.h rand.h queue.h
//		$(GCC)  -c mm1.cc
//
// driver.o:		driver.cc system.h snode.h olist.h rand.h queue.h
//		$(GCC)  -c driver.cc
//
// driverE.o:		driverE.cc system.h snode.h olist.h rand.h queue.h
//		$(GCC)  -c driverE.cc
//
// ************************************************************************ 

using namespace std;
#include <iostream>
#include "mm1.h"

main()
{

   char filename[50];          //name of file for data analysis
   ofstream dataOut;           //output stream object for data analysis file
   int numReps;
   double meanIAT, meanST;
   int numCusts;

   cout << "Enter name of file for data analysis: ";
   cin >> filename;
   dataOut.open(filename); 

   cout << endl << endl << endl;
   cout << "Simulation of a Single Server Queuing System -- " << endl;
   cout << "Interarrival times and service times are " << endl;
   cout << "exponentially distributed." << endl << endl;
   cout << "Enter the mean interarrival time: ";
   cin >> meanIAT;
   cout << endl;
   cout << "Enter the mean service time: ";
   cin >> meanST;
   cout << endl;
   cout << "Enter the number of customers: ";
   cin >> numCusts;
   cout << endl << endl;

   cout << "Enter the number of times to simulate: ";
   cin >> numReps;
   cout << endl << endl;

   for (int i = 0; i < numReps; i++)
     {
       mm1System SingleServerQ;
       SingleServerQ.initialize(meanIAT, meanST, numCusts);
       SingleServerQ.simulate();
       SingleServerQ.printReport(&dataOut);
     }
}

