//************************  FILE:  mm1.cc  ***********************************
//
// This file contains the definitions for the class mm1System
//
// Programmer:   J. Ingram
// Date:  January 22, 1997
// Updated:  May 27, 1997 to use the class simNode for list nodes
// Updated:  September 4, 2000 -- added scheduleDeparture(simNode *) and
//           updated the documentation
// Updated:  January 13, 2003 to overload initialize, adding a version
// with system parameters as arguments; and to overload printReport,
// allowing a report to be printed to a file in addition to standard
// output.
//
// Usage:  User-defined auxiliary files needed:
//
//              system.h,  the declaration file for the classes System
//                         and mm1System
//              olist.h,   the declaration file for the class OrderedList
//              queue.h,   the declaration file for the Queue class
//              snode.h,   the declaration file for the simNode class
//              rand.h,    the declaration file for the RandNum class
//
// ****************************************************************************
 
#include "mm1.h"

using namespace std;
#include <iostream>
#include "rand.h"

// ========================================================================
// Default constructor for an mm1System
//
// PRE:  The eventList and customerQ are empty.  The systemTime, number
// of customers who have completed their delay (numCustomers), the maximum
// number of customers through the system (maxCustomers), the total delay
// in the system (totalDelay), and the parameters for the random number
// objects (iat and serviceTime) are undefined.
//
// POST: The systemTime is 0.0 (set by the System constructor);
// the server status is IDLE; the number of customers who have
// completed their delay and the total delay are set to 0.  All other
// data members remain undefined.
// ========================================================================
mm1System::mm1System(): System()
{  
   serverStatus = IDLE;
   numCustomers = 0;
   totalDelay = 0.0;
 }


// ===========================================================================
// initialize() reads in (from standard input) the following system parameters
// and streams for generating random numbers:
//     --  the mean interarrival time for customers
//     --  a stream number for generating interarrival times
//     --  the mean service time for customers
//     --  a stream number for generating service times
//     --  the number of customers 
//
// PRE:  eventList and customerQ have been initialized to empty;
// systemTime is 0; the number of customers who have completed their
// delay (numCustomers) is 0; the total delay in the system
// (totalDelay) is 0.  The random number (type RandNum) objects (iat 
// and serviceTime) exist but their parameters are not defined.  The
// number of customers to simulate (maxCustomers) is undefined.
//
// POST:  The parameters for the random number objects (the means and
// stream numbers) are defined; maxCustomers is defined.  The eventList
// contains the first arrival event.
// ===================================================================
void mm1System::initialize()
{
   double mean;   //mean value for interarrival times and service times
   int stream;    //stream numbers for random number generators

   // Read input parameters and define random numbers
   cout << "Enter the mean interarrival time for customers: ";
   cin >> mean;
   cout << "Enter a stream number (1 - 100) for interarrivals: ";
   cin >> stream;
   RandNum exp(stream, mean);  
   iat = exp;
   cout << "Enter the mean service time for customers: ";
   cin >> mean;
   cout << "Enter a stream number (1 - 100) for service times: ";
   cin >> stream;
   RandNum service(stream, mean);
   serviceTime = service;
   cout << "Enter the number of customers to simulate: ";
   cin >> maxCustomers;

   // Schedule the first arrival
   simNode * firstArrival = new simNode(systemTime + iat.nextNum(), ARRIVAL);
   scheduleEvent(firstArrival);
 }


// ===========================================================================
// initialize(double meanIAT, double meanST, int maxCusts) 
// reads in stream numbers for interarrival times and service times and
// generates the random number objects from those streams and the
// system parameters sent as arguments.
//
// PRE:  eventList and customerQ have been initialized to empty;
// systemTime is 0; the number of customers who have completed their
// delay (numCustomers) is 0; the total delay in the system
// (totalDelay) is 0.  The random number (type RandNum) objects (iat 
// and serviceTime) exist but their parameters are not defined.  The
// number of customers to simulate (maxCustomers) is undefined.
//
// POST:  The parameters for the random number objects (the means and
// stream numbers) are defined; maxCustomers is defined.  The eventList
// contains the first arrival event.
// ===================================================================
void mm1System::initialize(double meanIAT, double meanST, int maxCusts)
{
   double mean;   //mean value for interarrival times and service times
   int stream;    //stream numbers for random number generators

   // Read stream numbers and define random numbers
   cout << "Enter a stream number (1 - 100) for interarrivals: ";
   cin >> stream;
   RandNum exp(stream, meanIAT);  
   iat = exp;


   cout << "Enter a stream number (1 - 100) for service times: ";
   cin >> stream;
   RandNum service(stream, meanST);
   serviceTime = service;

   //Initialize maximum number of customers to go through the system
   maxCustomers = maxCusts;

   // Schedule the first arrival
   simNode * firstArrival = new simNode(systemTime + iat.nextNum(), ARRIVAL);
   scheduleEvent(firstArrival);
 }


// =======================================================================
// continueSim() returns TRUE (1) if the simulation should continue (fewer
// than the maximum number of customers have completed their delay) and
// returns FALSE (0) if the simulation should stop.
// =======================================================================
int mm1System::continueSim()
{
   return (numCustomers < maxCustomers);
}


// =======================================================================
// updateState(Event *) calls the appropriate event routine to update
// the state of the system based on the type of the current event.
//
// PRE:  The state variables and statistical counters reflect the state
// of the system before the current event occurs.  systemTime contains
// the current simulation time; eventList contains events that have
// been scheduled (but have not yet occurred) earlier in the simulation.
// The argument is a pointer to the current event.
//
// POST:  The state variables and statistical counters reflect the
// state of the system as a result of the current event having happened.
// The eventList, in addition to the events on it in the pre-condition,
// contains events that were scheduled as a result of the current event.
// Specific changes are detailed in the event routines.
// =======================================================================
void mm1System::updateState(simNode * currentEvent)
{
   // call the appropriate event routine based on the type 
   // of the current event
   switch (currentEvent->getType())
     {
     case ARRIVAL:    processArrival(currentEvent);
       break;
     case DEPARTURE:  processDeparture(currentEvent);
       break;
     default:         cout << "Error -- Invalid Event Type" << endl;
     };
}


// ========================================================================
// processArrival(simNode *) updates the state of the system when an
// arrival event occurs.
//
// State Variables -  PRE:  serverStatus and customerQ are defined
//                    POST: serverStatus = BUSY; if serverStatus was 
//                          BUSY (pre), customerQ contains one more
//                          customer (the current arrival)
//                         
// Simulation Control Variables
//                    PRE:  eventList is defined and is either empty
//                          (if the serverStatus is IDLE) or contains
//                          the departure event for the customer being
//                          served
//                    POST: if the serverStatus was IDLE, a
//                          departure event for the current customer  
//                          is added to eventList; the next arrival
//                          event is added to the eventList
//
// Statistical Counters
//                    PRE:  totalDelay is defined and contains the delay
//                          of all customers who have completed their delay;
//                          customerQ statistics are defined 
//                    POST: totalDelay is unchanged; customerQ statistics
//                          are updated (by the Queue object) if the current
//                          customer is added to the queue.
// =========================================================================
void mm1System::processArrival(simNode * currentArrival)
{
    // Schedule the next arrival
    simNode * nextArrival = new simNode(systemTime + iat.nextNum(), ARRIVAL);
    scheduleEvent(nextArrival);

    // Process the current arrival
    if (serverStatus == IDLE)
      { 
	// Update the server status and schedule the departure of
	// this customer (change the node to a departure event) 
	serverStatus = BUSY;
	numCustomers++;
	scheduleDeparture(currentArrival);
      }
    else
      {  
	// Put the current arrival on the server's queue
	customerQ.enqueue(currentArrival, systemTime);
      };
  }


// =====================================================================
// processDeparture (simNode *) updates the state of the system when a
// departure event occurs.
//
// State Variables -  PRE:  serverStatus = BUSY; customerQ is defined
//                    POST: if customerQ was empty (pre), it remains
//                          empty and serverStatus is IDLE
//                          if customerQ was not empty(pre), a customer
//                          is removed from customerQ and serverStatus is
//                          unchanged (remains BUSY)
// Simulation Control Variables
//                    PRE:  eventList contains the arrival event for the
//                          next customer to arrive in the system
//                    POST: if the customerQ was not empty (pre), a
//                          departure event for the first customer in 
//                          the queue is added to eventList
// Statistical Counters
//                    PRE:  totalDelay is defined and contains the delay
//                          of all customers who have completed their delay;
//                          customerQ statistics are defined
//                    POST: totalDelay is updated to include the delay of
//                          the customer removed from the queue if there
//                          was a customer in the queue; customerQ statistics
//                          are updated by the Queue object
// =========================================================================
void mm1System::processDeparture(simNode * Depart)
{  
    delete Depart;

    if (customerQ.empty())   
      serverStatus = IDLE;
    else
      {  
	// "serve" the next customer in line by removing him/her
	// from the queue and scheduling the departure (putting
	// on the event list as a departure event to occur after
	// the service time is complete); addthat customer's
	// delay in the queu to the totalDelay and increment
	// the number of customers who have completed their delat 
	simNode * nextServed = customerQ.dequeue(systemTime);
	totalDelay += systemTime - nextServed->getTime();
	numCustomers++;
	scheduleDeparture(nextServed);
      };
  }


// ==========================================================================
// scheduleDeparture(simNode * customer) makes the customer node a DEPARTURE
// event and calls scheduleEvent to put it on the eventlist
//
// Parameters --      PRE:  customer points to a simNode for the customer
//                          whose departure is to be scheduled
//                    POST: the time field of the customer node has been
//                          set to the current simulation time plus a 
//                          random service time; the type field has been set
//                          to DEPARTURE; the node has been added to the
//                          eventList
//
// State Variables -  State variables are unchanged
//
// Simulation Control Variables 
//                    PRE:  eventList is defined and contains events that
//                          have been previously scheduled but have not yet
//                          occurred
//                    POST: the departure event for the parameter customer
//                          has been added to the eventList
//
// Statistical Counters - unchanged 
// ==========================================================================
void mm1System::scheduleDeparture(simNode * customer)
{
    double departTime; // departure time of the customer

    departTime = systemTime + serviceTime.nextNum();
    customer->setTime(departTime);
    customer->setType(DEPARTURE);
    scheduleEvent(customer);
}



// =================================================
// printReport() prints the simulation results
// =================================================
void mm1System::printReport()
{
  System::printReport();
  cout << "\nSingle Server Queueing System\n";
  cout << "=============================\n";
  cout << endl;
  cout << "Mean Interarrival Time of Customers: " << iat.mean() << endl;
  cout << "Mean Service Time: " << serviceTime.mean() << endl;
  cout << "Number of Customers who completed their delay: " << numCustomers;
  cout << endl << endl;
  cout << "Mean Delay in the Queue: " << totalDelay/numCustomers << endl;
  
  double avgNumInQ = customerQ.totalArea()/systemTime;
  cout << "Time Average Number in the Queue: " << avgNumInQ << endl;
  cout << endl;
}


// =================================================
// printReport(ofstream *) prints the simulation 
// results to standard output and a file. 
// =================================================
void mm1System::printReport(ofstream * dataOut)
{
  System::printReport();
  cout << "\nSingle Server Queueing System\n";
  cout << "=============================\n";
  cout << endl;
  cout << "Mean Interarrival Time of Customers: " << iat.mean() << endl;
  cout << "Mean Service Time: " << serviceTime.mean() << endl;
  cout << "Number of Customers who completed their delay: " << numCustomers;
  cout << endl << endl;
  cout << "Mean Delay in the Queue: " << totalDelay/numCustomers << endl;
  
  double avgNumInQ = customerQ.totalArea()/systemTime;
  cout << "Time Average Number in the Queue: " << avgNumInQ << endl;
  cout << endl;
  
  *dataOut << iat.getStream() << "    " << serviceTime.getStream();
  (*dataOut).precision(4);
  (*dataOut).setf(ios::fixed, ios::floatfield);
  (*dataOut).width(18);
  *dataOut << totalDelay/numCustomers;
  (*dataOut).width(18);
  *dataOut << avgNumInQ << endl;  
}








