// **********************  FILE:  mm1.h   **********************************
//
// This file contains the declarations for the class mm1System derived
// from the base class System.
//
// The mm1System class is an mm1 queuing system.  That is, it describes
// a system with one server and one queue of "customers" the server is
// serving.  In an mm1 queuing system it is assumed that the interarrival
// times of customers into the system are exponentially distributed and
// that the service times of the customers are exponentially distributed.
// The means for these distributions are input parameters for the system.
//
// Performance measures computed:  average delay in the queue and the
// average number in the queue
//
// Stopping condition for the simulation:  when a prespecified number of
// customers have completed their delay in the system (the number to 
// complete their delay is an input to the system)
//
// Input to the system:  
//        -- mean interarrival time for customers in the system
//        -- a stream number (1 - 100) for generating random interarrivals
//        -- mean service time for customers
//        -- a stream number (1 - 100) for generating random service times
//        -- the number of customers to simulate
// 
//
// Needs the following files:  system.h, snode.h, olist.h, queue.h, rand.h
//
// ************************************************************************
#ifndef MM1_DEF
#define MM1_DEF

using namespace std;

#include "system.h"
#include "olist.h"
#include "rand.h"
#include "queue.h"
#include <fstream>



// ****************************************
// *******  mm1System Declarations  *******
// ****************************************
class mm1System: public System
{
   typedef enum {ARRIVAL,DEPARTURE} eventType;
  
   typedef enum {IDLE, BUSY} statusType;

   public:
      mm1System();                     // Construct an mm1 queuing system
                                       // with an idle server and no 
                                       // customers having completed their
                                       // delay
      void initialize();               // Reads in system parameters
                                       // and sets up random number generators.
      void initialize(double meanIAT,
         double meanST, int maxCustomers);               
                                       // Set system parameters and 
                                       // random number generators.
                                       // Schedule the first arrival.
      void updateState(simNode *);     // Call the appropriate event routine
                                       // to update the state of the system
                                       // depending on the type event that 
                                       // occurred (given as the type field
                                       // of the simNode argument)
      void processArrival(simNode *);  // Event routine for an arrival event.
                                       // Updates the state of the system
                                       // when an arrival occurs, updates
                                       // the statistical counters, and
                                       // schedules the next arrival.
                                       // Schedules a future departure if
                                       // appropriate (if current arrival is
                                       // immediately served)
      void processDeparture(simNode *); // Event routine for a departure event.
                                       // Updates the state of the system
                                       // when a departure occurs, updates
                                       // the statistical counters.  Schedules
                                       // a future departure (if another
                                       // customer starts being served).
      void scheduleDeparture(simNode *); // Schedules a departure event.
      int continueSim();               // Returns TRUE (1) if the number of
                                       // customers who have completed their
                                       // delay is less than the number to be
                                       // simulated; otherwise returns FALSE
                                       // (0)
      void printReport ();             // Prints a report showing the system
                                       // statistics to standard output
      void printReport(ofstream *);    // Prints a report showing the
                                       // system statistics to standard 
                                       // output and a file.
   private:
      statusType serverStatus;         // Status of the server (BUSY or IDLE)
      Queue customerQ;                 // Queue for the customers
      RandNum iat;                     // Random number for generating
                                       // interarrival times of customers
      RandNum serviceTime;             // Random number for generating
                                       // service times of customers
      int numCustomers;                // Current number of customers who
                                       // have completed their delay in the
                                       // system (in the queue)
      int maxCustomers;                // The total number of customers to
                                       // simulate -- simulation stops when
                                       // maxCustomers have completed their
                                       // delay in the system
      double totalDelay;               // Statistical counter -- the total
                                       // time spent in the queue by all
                                       // customers
};

#endif
