// ******************** FILE:  queue.cc  ************************************
//
// This file contains the definitions for the Queue class.
//
// Programmer:  J. Ingram
// Date:   January 22, 1997
// Updated:  May 27, 1997 (to use the simNode class for node type
//
// Usage:  User-defined auxiliary files needed:
//
//              queue.h, the declaration file for the Queue class
//
// **************************************************************************  
#include "queue.h"

using namespace std;

#include <iostream>

// ***********************************
// *******  Queue Definitions  *******
// ***********************************

// ===========================================================
// Construct an empty queue
//
// PRE:  The Queue object is undefined
//
// POST: Data members are defined to indicate an empty queue
// that has never been modified (area and lasttime are 0)
// ===========================================================
Queue::Queue()
{
    niq = 0;
    area = 0.0;
    lasttime = 0.0;
    front = NULL;
    rear = NULL;
}

// ====================================================================
// enqueue adds a node to the rear of the queue
//
// PRE:  The queue is defined with the nodes in order by the time
//       they were added to the list (the most recent one at the end
//       of the list).  The variable niq contains the current number in
//       the queue, area contains the total area under the number in
//       queue curve as of the last time the queue changed size, 
//       lasttime contains the time that the queue last changed size.
//       The first argument is a pointer to the node to be inserted
//       in the queue; the second argument is the current time.
//
// POST: The new node is at the end of the queue; niq contains the number
//       in the queue (one more than before), area contains the total
//       area under the number in queue curve (the total time for all nodes
//       that have been in the queue), lasttime contains the current
//       time (the value of the second argument).
// =========================================================================
void Queue::enqueue(simNode * node, double systime)
{
    // Update queue statistics
    area += niq*(systime - lasttime);
    niq++;
    lasttime = systime;

    // Add node to rear of the queue
    if (rear != NULL)
       rear->next = node;
    else
       front = node;
    rear = node;
    node->next = NULL;
}


// ===================================================================
// dequeue removes the first node from the queue and returns a pointer
// to it. 
//
// PRE:  The queue is defined with the nodes in order by the time
//       they were added to the list (the most recent one at the end
//       of the list).  The variable niq contains the current number in
//       the queue, area contains the total area under the curve as of
//       the last time the queue changed size, lasttime contains the
//       time that the queue last changed size.
//       The argument (double) is the current time.
//
// POST: The front node in the queue is the node that was previously
//       second; the variable niq contains the current number in the
//       queue (one less than previously), area contains the total area 
//       under the number in queue function, lasttime contains the current
//       system time (the value of the argument).
// ========================================================================
simNode * Queue::dequeue (double systime)
{
    simNode * tempnode;    // temporary holder for node removed from the queue

    tempnode = front;
    
    if (tempnode == NULL)
      cerr << "Error -- attempt to remove node from empty queue.\n";
    else {
       // Update queue statistics
       area += niq*(systime - lasttime);
       niq--;
       lasttime = systime;

       // The front of the queue moves to the next node
       front = front->next;

       // Adjust pointers if the queue is empty 
       if (front == NULL)
          rear = NULL;
     }
    return (tempnode);
}


// ==================================================================
// empty returns TRUE (1) if the queue is empty; otherwise it returns
// FALSE (0)
// ==================================================================
int Queue::empty ()
{
    return (niq == 0);
}


// =================================================================
// totalArea returns the total area under the number in queue curve
// =================================================================
double Queue::totalArea() 
   {return area;}


// ================================================================
// display prints the data members of the Queue class and prints
// the time field of the nodes in the queue
// ================================================================
void Queue::display ()
{
   simNode * ptr;      // ptr is used to traverse the nodes in the queue
   ptr = front;
   cout << "Number in the queue: " << niq << endl;
   cout << "Area: " << area << endl;
   cout << "Nodes in the Queue: " << endl;

   // traverse queue, displaying each node
   ptr = front;
   while (ptr != NULL)  {
      ptr->display();
      ptr = ptr->next;
    }
 }




