// ********************  FILE:  system.cc  *********************************
//
// This file contains the definitions for the System class.
//
// Programmer:  J. Ingram
// Date:  January 22, 1997
// Updated:  May 27, 1997 to use the class simNode for list nodes
//
// Usage:   User-defined auxiliary files needed for the class System:
//
//            system.h, the declaration file for the System class
//            olist.h,  the declaration file for the OrderedList class
//            snode.h,  the declaration file for the simNode class
//            queue.h,  the declaration file for the Queue class
//            rand.h,   the declaration file for the RandNum class
//
// **************************************************************************
#include "system.h"

using namespace std;
#include <iostream>


// ***********************************
// *******  System Definitions *******
// ***********************************

// ===========================================================
// Constructor for a basic system -- systemTime is set to 0
// ===========================================================
System::System()
 { systemTime = 0.0; }


// ====================================
// Return the current simulation time
// ====================================
double System::getTime()
{ return systemTime; }


// =============================================================
// Default initialization of a system -- Does nothing.  Derived
// classes should define their own initializations.
// =============================================================
void System::initialize()
{ }


// ===============================================================
// Simulate a system using the event scheduling approach -- the
// main simulation engine for simulating a system.
//
// Algorithm:
// Repeat the following until the stopping criterion is met:
//      --- get the next event off the event list
//      --- update the system clock to the time of the event
//      --- update the state of the system based on the
//          occurrence of the event
//
// PRE:  The system is in its initial state (which depends on 
// the initializations and constructors in the derived class).  The 
// eventList is non-empty (at least one event has been scheduled).
// Statistical counters (defined in the derived class) have been 
// initialized.
//
// POST: The stopping criterion for the simulation has been
// met.  systemTime is the length (in time units) of the
// simulation.  State variables and statistical counters reflect
// the state of the system and the performance measures for the 
// simulation at the time the simulation stops.
// =============================================================    
void System::simulate()
{  
   simNode * currentEvent;   // pointer to the event currently
                             // "happening"
   while (continueSim())
     {
        currentEvent = eventList.getFirst();
        systemTime = currentEvent->getTime();
        updateState(currentEvent);
      };
 }


// ======================================================= 
// scheduleEvent schedules an event by having a node for
// the event inserted into the eventList.
//
// PRE: eventList contains future events that have
// previously been scheduled but that have not yet
// occurred (their time of occurrence is greater than
// or equal to the current systemTime).  The argument
// is a pointer to an event (type simNode) that is to happen
// some time in the future (or the current systemTime)
//
// POST:  The futureEvent has been added to the eventList
// if its time is greater than or equal to the current
// system time.
// =======================================================
void System::scheduleEvent(simNode * futureEvent)
{
   if (futureEvent->getTime() >= systemTime)
      eventList.insert(futureEvent);
   else   {
      cout << "Error:  Attempt to schedule an event before the current";
      cout << " simulation time." << endl;
      cout << "System time: " << systemTime << endl;
      cout << "Event time: " << futureEvent->getTime() << endl;
   };
 } 


// ==========================================================
// printReport prints a basic report containing the current
// simulated time
// ==========================================================
void System::printReport()
{
   cout << endl;
   cout << "System Report" << endl;
   cout << "=============" << endl << endl;
   cout << "System Time: " << systemTime << endl;
   cout << endl;
}

 






















